#! /usr/bin/python
## -*- coding: utf-8 -*-

##
##  HARK Middleware
##  Copyright (C) 2013-2014 by Honda Research Institute Japan Co., Ltd.
##
##  $Id:$
##

from harkmw.HarkError import HARKError
from harkmw.logic.link import Link

import os
debug = (bool(os.environ['HARK_DEBUG'])) if 'HARK_DEBUG' in os.environ else False

class Node :
    """ Base-class of Network, NativeNode, Node
    """
    def __init__(self) :
        self.name    = None
        self.type    = None
        self.network = None
        self.count   = -1

    def request(self, process):
        look_windows = [(0, 0)]
        for (output_name, output) in self.outputs.items() :
            for output_proxy in output:
                look_windows.append(output_proxy.get_look_window())
        look_window = max(look_windows)

        if debug:
            print("Node: request: name={}, window={}, windows={}".format(self.name, look_window, look_windows))
        for (output_name, output) in self.outputs.items() :
            self.set_output_look_window(output_name, look_window)
        for (input_name, input) in self.inputs.items():
            input.request(process)

    def initialize(self, process) :
#        node_delay = self.get_delay()
#        #print("Initialize: node, name={}, delay={}".format(self.name, node_delay))
        for input in self.inputs.values() :
#            proxy_delay = input.get_delay()
#            if proxy_delay < node_delay :
#                #print("Set lookAhead: {}, {}".format(node_delay - proxy_delay, input))
#                input.set_look_window(node_delay - proxy_delay, 0)
            input.initialize(process)

    def release(self) :
        pass

    def is_input_ready(self, count) :
        """ ループ count のデータを生成するための入力データは整っているか？
        """
        return True # 既定動作は常に True

    def push_data(self, input, count, process, internal=False, outer_count=-1) :
        """
        ループ count のデータが input に入力された場合にコールされる
        処理された場合は True を返す。他の input が準備できないない場合など、処理されない場合は False を返す。
        """
        if debug:
            print("Try push data for Node [{}/{}], count [{}/{}], #inputs={}, #outputs={}, ({})".format(self.type, self.name, count, self.count, len(self.inputs), len(self.outputs), self))
        
#       if input is not None :
#           input.count = count

#       if not internal and self.type == "Collector" and self.name[-6:] == "_input" :
#           return self.network.push_data(input, count, process)

        if self.count < count:
            target_count = self.count + 1
        else:
            target_count = count
        if debug:
            print("target-count: {}".format(target_count))
        for i in self.inputs.values() :
            if debug:
                print("Input[{}]: count={},{},{}".format(i, i.count, self.count, count))
#            if False and not i.is_count_available(target_count) :
#                """ まだ必要なデータが整っていない入力が存在する """
#                if debug:
#                    print("Not enough data available. ({} < {} + {}): {}".format(count, target_count, i.get_look_window(), i))
#                return target_count - 1
#            else:
#                if debug:
#                    print("Enough data available. ({} >= {} + {}): {}".format(count, target_count, i.get_look_window(), i))

#        if self.count >= count:
#            print("re-visit")

        if self.name[-7:] == '_output':
            self.input_count = target_count
        self.count = target_count

        maximum_look_ahead = 0
        for (output_name, output) in self.outputs.items() :
            for output_proxy in output:
                maximum_look_ahead = max(maximum_look_ahead, output_proxy.get_look_window()[0])

        minimum_target_count = target_count
        for (output_name, output) in self.outputs.items() :
            if debug:
                print("output for {}: {}".format(self.name, output_name))
            if self.name[-7:] == '_output':
                self.process(output_name, target_count, input_count=target_count, output_count=outer_count)
            elif self.name[-6:] == '_input':
#                if self.network.network_type == 'iterator':
#                    print("input node for iterator network")
#                else:
#                    print("input node for non-iterator network: {}".format(self.network))
                    self.process(output_name, target_count, input_count=outer_count, output_count=target_count)
            else:
                self.process(output_name, target_count)

            if self.name[-7:] != '_output': # and not (self.name[-6:] == '_input' and self.network.network_type == 'iterator'):
                for output_proxy in output :
                    if debug:
                        print("Node.push_data: Push: {}: {}/{}: {}".format(self.name, count, target_count, output_proxy))
                    look_ahead = output_proxy.get_look_window()[0]
                    diff_look_ahead = maximum_look_ahead - look_ahead
                    if debug:
                        print("diff_look_ahead: target_count={}, diff={}, self={}, max={}, output_proxy={}".format(target_count, diff_look_ahead, output_proxy.get_look_window(), maximum_look_ahead, output_proxy))
                    if True or (target_count >= look_ahead and output_proxy is not None):
                        if debug:
                            print("do push_data")
                        #minimum_target_count = min(minimum_target_count, output_proxy.push_data(target_count - look_ahead, process))
                        minimum_target_count = min(minimum_target_count, output_proxy.push_data(target_count, process))
                    else:
                        if debug:
                            print("skip push_data")

        return minimum_target_count

    def get_input_count(self) :
        raise NotImplementedError("")

    def get_output_count(self) :
        raise NotImplementedError("")

    def get_data(self, terminal_name, count) :
        raise NotImplementedError("")

    def get_delay(self) :
        delay = 0
        for input in self.inputs.values() :
            delay = max(delay, input.get_delay())
        return delay

    def get_topic(self) :
        if self.topic is None :
            return "%s/%s" % (self.network.get_topic(), self.name)
        else :
            return self.topic

# end of file
