/*
 */

#include "libharkmw.h"

#include <hark-base/BufferedNode.h>
#include <hark-base/Vector.h>
#include <hark-base/Matrix.h>
#include <hark-base/Map.h>

#include "type.h"

///* From harkfd/Source.h */
//namespace HARK{
//    class Source : public HARK::Object {
//    public:
//        float x[3];
//        float power;
//        int id;
//        float remaining;
//
//        Source();
//        Source(const Source& src);
//        float angle(const float* y) const;
//        float angle(const Source& src);
//        virtual HARK::ObjectRef clone();
//        virtual void printOn(std::ostream& out) const;
//        void readFrom(std::istream &in);
//        friend std::istream& operator>>(std::istream &in, Source& src);
//        ~Source(){}
//    };
//}
///* From harkfd/Source.h [end] */

namespace HARKMW
{
    PyObject *
    toPythonObject(HARK::ObjectRef &obj)
    {
//        fprintf(stdout, "Type: %s\n", typeid(*obj).name());
//        fprintf(stdout, "  ? %s\n", typeid(HARK::Vector<HARK::ObjectRef>).name());
//        fprintf(stdout, "  ? %s\n", typeid(HARK::Map<int, HARK::ObjectRef>).name());
//        fprintf(stdout, "  ? %s\n", typeid(HARK::Matrix<float>).name());
//        fprintf(stdout, "  ? %s\n", typeid(HARK::Matrix<HARK::Complex<float> >).name());
//        fprintf(stdout, "  ? %s\n", typeid(HARK::Source).name());
//        fprintf(stdout, "  ? %s\n", typeid(bool).name());
//        fprintf(stdout, "  ? %s\n", typeid(HARK::NetCType<bool>).name());
//        fprintf(stdout, "  ? %s\n", typeid(HARK::NetCType<HARK::Bool>).name());
//        fprintf(stdout, "  ? %s\n", typeid(HARK::String).name());
//        fprintf(stdout, "  ? %s\n", typeid(std::string).name());
        if(typeid(*obj) == typeid(bool) ||
           typeid(*obj) == typeid(HARK::NetCType<bool>) ||
           typeid(*obj) == typeid(HARK::NetCType<HARK::Bool>)){
            
            if(HARK::dereference_cast<bool>(obj)){
//                fprintf(stdout, "Returning true\n");
                Py_RETURN_TRUE;
            }else{
//                fprintf(stdout, "Returning false\n");
                Py_RETURN_FALSE;
            }

        }else if(typeid(*obj) == typeid(std::string)){
            return PyStr_FromString(HARK::dereference_cast<std::string>(obj).c_str());

        }else if(typeid(*obj) == typeid(HARK::String)){
            return PyStr_FromString(static_cast<HARK::String *>(&(*obj))->c_str());

//        }else if(typeid(*obj) == typeid(HARK::Source)){
//            HARK::Source *value = static_cast<HARK::Source *>(&(*obj));
//            return Py_BuildValue("{sssfsfsfsfsisf}",
//                                 "_type", "HARK::Source",
//                                 "x", value->x[0],
//                                 "y", value->x[1],
//                                 "z", value->x[2],
//                                 "power", value->power,
//                                 "id", value->id,
//                                 "remaining", value->remaining);

        }else if(typeid(*obj) == typeid(HARK::Vector<HARK::ObjectRef>)){
            HARK::Vector<HARK::ObjectRef> &value = *(static_cast<HARK::Vector<HARK::ObjectRef> *>(&(*obj)));
            PyObject *pobj = PyList_New(value.size());

            size_t i = 0;
            for(HARK::Vector<HARK::ObjectRef>::iterator it = value.begin(); it != value.end(); ++it){
                HARK::ObjectRef &v = *it;
                PyList_SET_ITEM(pobj, i++, toPythonObject(v));
            }

            return pobj;

        }else if(typeid(*obj) == typeid(HARK::Matrix<float>)){
            HARK::Matrix<float> &value = *(static_cast<HARK::Matrix<float> *>(&(*obj)));
            PyObject *pobj = PyList_New(value.nrows());

            size_t i;
            for(i = 0; i < value.nrows(); i++){
                PyObject *robj = PyList_New(value.ncols());
                size_t j;
                for(j = 0; j < value.ncols(); j++){
                    PyList_SET_ITEM(robj, j, Py_BuildValue("f", value(i, j)));
                }
                PyList_SET_ITEM(pobj, i, robj);
            }

            return pobj;

        }else if(typeid(*obj) == typeid(HARK::Matrix<std::complex<float> >)){
            HARK::Matrix<std::complex<float> > &value = *(static_cast<HARK::Matrix<std::complex<float> > *>(&(*obj)));
            PyObject *pobj = PyList_New(value.nrows());

            size_t i;
            for(i = 0; i < value.nrows(); i++){
                PyObject *robj = PyList_New(value.ncols());
                size_t j;
                for(j = 0; j < value.ncols(); j++){
                    PyList_SET_ITEM(robj, j, Py_BuildValue("(ff)", value(i, j).real(), value(i, j).imag()));
                }
                PyList_SET_ITEM(pobj, i, robj);
            }

            return pobj;

        }else if(typeid(*obj) == typeid(HARK::Matrix<HARK::Complex<float> >)){
            HARK::Matrix<HARK::Complex<float> > &value = *(static_cast<HARK::Matrix<HARK::Complex<float> > *>(&(*obj)));
            PyObject *pobj = PyList_New(value.nrows());

            size_t i;
            for(i = 0; i < value.nrows(); i++){
                PyObject *robj = PyList_New(value.ncols());
                size_t j;
                for(j = 0; j < value.ncols(); j++){
                    PyList_SET_ITEM(robj, j, Py_BuildValue("(ff)", value(i, j).real(), value(i, j).imag()));
                }
                PyList_SET_ITEM(pobj, i, robj);
            }

            return pobj;

//        }else if(typeid(*obj) == typeid(HARK::Map<int, HARK::ObjectRef>)){
//            Py_RETURN_NONE;

        }else{
            std::cerr << "Failed to convert C++ object to Python object: ";
            obj->printOn(std::cerr);
            std::cerr << std::endl;
            Py_RETURN_NONE;
        }
    }

    HARK::ObjectRef
    fromPythonObject(PyObject *obj)
    {
        HARK::ObjectRef o;

        if(PyStr_Check(obj)){
            o = HARK::ObjectRef(new HARK::String(PyStr_AsUTF8(obj)));

        }else if(PyList_Check(obj)){
            size_t list_size = PyList_Size(obj);
            PyObject *list_head = PyList_GetItem(obj, 0);

            if(PyList_Check(list_head)){
                size_t list2_size = PyList_Size(list_head);
                PyObject *list2_head = PyList_GetItem(list_head, 0);
                
                if(PyTuple_Check(list2_head)){
                    // May be Matrix<Complex<float>>
                    HARK::Matrix<std::complex<float> > *mat = new HARK::Matrix<std::complex<float> >(list_size, list2_size);

                    int i, j, n, m;
                    for(i = 0, n = PyList_Size(obj); i < n; i++){

                        PyObject *p = PyList_GetItem(obj, i);
                        for(j = 0, m = PyList_Size(p); j < m; j++){
                            float vr, vi;
                            PyArg_Parse(PyList_GetItem(p, j), "(ff)", &vr, &vi);
                            (*mat)(i, j) = std::complex<float>(vr, vi);
                        }
                    }

                    o = HARK::ObjectRef(mat);
                    
                }else{
                    // May be Matrix<float>
                    HARK::Matrix<float> *mat = new HARK::Matrix<float>(list_size, list2_size);

                    int i, j, n, m;
                    for(i = 0, n = PyList_Size(obj); i < n; i++){

                        PyObject *p = PyList_GetItem(obj, i);
                        for(j = 0, m = PyList_Size(p); j < m; j++){
                            float v;
                            PyArg_Parse(PyList_GetItem(p, j), "f", &v);
                            (*mat)(i, j) = v;
                        }
                    }

                    o = HARK::ObjectRef(mat);
                }

            }else if(PyTuple_Check(list_head)){
                // May be Complex<float>
                o = HARK::ObjectRef(new HARK::Complex<float>(list_size));

            }else{
                o = HARK::ObjectRef(new HARK::String(PyStr_AsUTF8(obj)));
            }

        }else{
            PyObject *str = PyObject_Str(obj);
            std::cerr << "Failed to convert Python object to C++ object: ";
            std::cerr << PyStr_AsUTF8(str);
            std::cerr << std::endl;
            Py_DECREF(str);
        }

        return o;
    }

}

/* end of file */
