#
#  Use as:
#    $ python setup.py build
#    $ python setup.py install
#

import os
from setuptools import setup, Extension, find_packages
from distutils.sysconfig import get_python_inc
import subprocess

if __name__ == '__main__':
    py_incdir = os.path.join(get_python_inc(plat_specific = 1));
    hb_libdir = subprocess.check_output('pkg-config hark-base --libs-only-L'.split()).decode().strip()[2:]
    hb_libname = subprocess.check_output('pkg-config hark-base --libs-only-l'.split()).decode().strip()[2:]
    hb_incdir = subprocess.check_output('pkg-config hark-base --cflags-only-I'.split()).decode().strip()[2:]
    hb_incdir = hb_incdir[:hb_incdir.rfind('/')]
    print("incdir =", [py_incdir, hb_incdir, 'harkmw/src'])
    #hb_libdir = '/usr/local/lib'
    #hb_libdname = 'hark-base'
    #hb_incdir = '/usr/local/include/hark/hark-base'

    setup(
        name         = 'harkmw',
        version      = '3.0.3',
        description  = 'Data-flow middleware for HARK',
        author       = 'HARK support team',
        author_email = 'support@hark.jp',
        url          = 'https://www.hark.jp',
        license      = 'HARK License',
        entry_points = {
            'console_scripts':[
                'harkmw=harkmw.main:main',
                'batchflow=harkmw.main:main'
            ]
        },
        packages         = find_packages(),
#       install_requires = ['paho-mqtt>=1.3'],
        tests_require    = [],
        test_suite       = 'nose.collector',
        include_package_data=True,
        zip_safe         = False,
        ext_modules = ([
            Extension('harkmwnative',
                      sources = ['harkmw/src/libmain.cc',
                                 'harkmw/src/proxy.cc',
                                 'harkmw/src/type.cc',
                                 'harkmw/src/node.cc',
                                 'harkmw/src/collector.cc',
                                 'harkmw/src/object.cc',
                      ],
                      include_dirs = [py_incdir, hb_incdir, 'harkmw/src'],
                      library_dirs = [hb_libdir],
                      runtime_library_dirs = [hb_libdir] if os.name !='nt' else None,
                      libraries    = [hb_libname],
                      extra_compile_args = ['-O2', '-std=c++11', '-include', 'sstream', '-D_hypot=hypot', '-g', '-Wno-write-strings', '-Wno-sign-compare'])
        ])
    )
